% PAL (Pareto Active Learning) Algorithm 
%
% Copyright (c) 2013 ETH Zurich
% 
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

function cell_out = classify_point(i, u_rt_obj1, u_rt_obj2, s_rt_obj1,s_rt_obj2,epsilon,data_range, not_p_all)
    prev_len = length(u_rt_obj2);
    range_obj1 = data_range{1};
    range_obj2 = data_range{2};
    
    not_p_all(i) = 1;
    
    area_optimistic_i = (u_rt_obj1(i)+(epsilon*range_obj1)) - s_rt_obj1(i);
    area_pessimistic_i = (u_rt_obj1(i)-(epsilon*range_obj1)) + s_rt_obj1(i);
    thrput_optimistic_i = (u_rt_obj2(i)-(epsilon*range_obj2)) + s_rt_obj2(i);
    thrput_pessimistic_i = (u_rt_obj2(i)+(epsilon*range_obj2)) - s_rt_obj2(i);
    
    u_rt_obj1 = removerows(u_rt_obj1,find((not_p_all)==1));
    u_rt_obj2 = removerows(u_rt_obj2,find((not_p_all)==1));
    s_rt_obj1 = removerows(s_rt_obj1,find((not_p_all)==1));
    s_rt_obj2 = removerows(s_rt_obj2,find((not_p_all)==1));

    removed = prev_len-length(s_rt_obj2);
    
    area_optimistic = (u_rt_obj1+(epsilon*range_obj1)) - s_rt_obj1;
    area_pessimistic = (u_rt_obj1-(epsilon*range_obj1)) + s_rt_obj1;
    thrput_optimistic = (u_rt_obj2-(epsilon*range_obj2)) + s_rt_obj2;
    thrput_pessimistic = (u_rt_obj2+(epsilon*range_obj2)) - s_rt_obj2;

	a_o_i = kron(area_optimistic_i,ones(length(u_rt_obj1),1));
	a_p_i = kron(area_pessimistic_i,ones(length(u_rt_obj1),1));
	t_o_i = kron(thrput_optimistic_i,ones(length(u_rt_obj1),1));
	t_p_i = kron(thrput_pessimistic_i,ones(length(u_rt_obj1),1));

	a1 = a_o_i < area_pessimistic;
	a2 = area_optimistic < a_p_i;
	t1 = t_o_i > thrput_pessimistic;
	t2 = thrput_optimistic > t_p_i;

	p = a2 & t2;
	np = a1 | t1;
	
	sum_pareto = sum(p);
	pareto = sum_pareto ==0;

	not_p = sum(np);
	not_p = not_p<length(u_rt_obj1);
    
    not_p = not_p & not(pareto);
    
    cell_out{1}=pareto;
    cell_out{2}=not_p;

 
