% PAL (Pareto Active Learning) Algorithm 
%
% Copyright (c) 2013 ETH Zurich
% 
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

function [pareto_hv_error_obj1,pareto_hv_error_obj2] = get_pareto_hv_error(real_pareto_set, pareto_01list, real_list_obj1, real_list_obj2, log_train)

    real_pareto_list_obj1 = real_pareto_set{1};
    real_pareto_list_obj2 = real_pareto_set{2};
    
    %find pareto sets of real values from predicted 01 list
    predicted_pareto_list_obj1 = -1;
    predicted_pareto_list_obj2 = -1;
        
    for i=1:length(pareto_01list)       
        if ((pareto_01list(i)==1))
         	predicted_pareto_list_obj1 = append(predicted_pareto_list_obj1,real_list_obj1(i));
            predicted_pareto_list_obj2 = append(predicted_pareto_list_obj2,real_list_obj2(i));
        end
    end
    
    if log_train == 1
        for i=1:length(real_pareto_list_obj1)   
            real_pareto_list_obj1(i) = (2^real_pareto_list_obj1(i));
            real_pareto_list_obj2(i) = (2^real_pareto_list_obj2(i));
        end
    end
    
    for i=1:length(real_pareto_list_obj1)   
        real_pareto_list_obj1(i) = log(real_pareto_list_obj1(i));
        real_pareto_list_obj2(i) = log(real_pareto_list_obj2(i));

    end

    if log_train == 1
        for i=1:length(predicted_pareto_list_obj1)   
            predicted_pareto_list_obj1(i) = (2^predicted_pareto_list_obj1(i));
            predicted_pareto_list_obj2(i) = (2^predicted_pareto_list_obj2(i));
        end
    end
    
    for i=1:length(predicted_pareto_list_obj1)   
        predicted_pareto_list_obj1(i) = log(predicted_pareto_list_obj1(i));
        predicted_pareto_list_obj2(i) = log(predicted_pareto_list_obj2(i));
    end

        
    
    [real_pareto_list_obj1,real_pareto_list_obj2,tmp_rest_list] = order_pareto(real_pareto_list_obj1,real_pareto_list_obj2,zeros(length(real_pareto_list_obj1),1));
    [real_pareto_list_obj1,real_pareto_list_obj2] = get_pareto_set_to_plot(real_pareto_list_obj1,real_pareto_list_obj2);
        
    [predicted_pareto_list_obj1,predicted_pareto_list_obj2,tmp_rest_list] = get_pareto(predicted_pareto_list_obj1,predicted_pareto_list_obj2,zeros(length(predicted_pareto_list_obj2),1));
    [predicted_pareto_list_obj1,predicted_pareto_list_obj2,tmp_rest_list] = order_pareto(predicted_pareto_list_obj1,predicted_pareto_list_obj2,zeros(length(predicted_pareto_list_obj1),1));

    [predicted_pareto_list_obj1,predicted_pareto_list_obj2] = get_pareto_set_to_plot(predicted_pareto_list_obj1,predicted_pareto_list_obj2);
    
    %finding common reference points
    max_real = max(real_pareto_list_obj1);
    max_predicted = max(predicted_pareto_list_obj1);
    if max_real>max_predicted
        max_obj1 = max_real;
    else
        max_obj1 = max_predicted;
    end
    
    min_real = min(real_pareto_list_obj2);
    min_predicted = min(predicted_pareto_list_obj2);
    if min_real<min_predicted
        min_obj2 = min_real;
    else
        min_obj2 = min_predicted;
    end
    
    max_obj1 = max_obj1 * 1.05;
    min_obj2 = min_obj2 - abs(min_obj2) * 0.05;
        
    range_obj1 = max_obj1 - min(real_pareto_list_obj1);
    range_obj2 = max(real_pareto_list_obj2) - min_obj2;

    %pass list of pareto points only
    hyperv_real = get_hypervolume(real_pareto_list_obj1, real_pareto_list_obj2, max_obj1, min_obj2);
    hyperv_prediction = get_hypervolume(predicted_pareto_list_obj1, predicted_pareto_list_obj2, max_obj1, min_obj2);

    %get hypervolume difference 
    hyperv_diff = (hyperv_real - hyperv_prediction);

    %get percentage error for area and freq
    pareto_hv_error_obj1 = (1-exp(-hyperv_diff/range_obj2))*100;
    pareto_hv_error_obj2 = (1-exp(-hyperv_diff/range_obj1))*100;
        

end