% PAL (Pareto Active Learning) Algorithm 
%
% Copyright (c) 2013 ETH Zurich
% 
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

function data_array = get_start_samples_new(data_array,length_training_start,num_features)
	data_array = data_array(randsample(1:length(data_array),length(data_array)),:);
	%len_data_array = length(data_array(:,bflies_index));

	features=data_array(:,1:num_features);
	length_features = length(features);

	selected_samples=[];
	rest_data=[];

	% calculate dists
	feature_array = []
	for i=1:length(features)
		feature_array=[feature_array;features(i,:)];
	end

	dists_vec = pdist(feature_array);
	dists_indices_fst = [];
	dists_indices_snd = [];
	for i=1:length(features)-1

		for j=i:length(features)
			dists_indices_fst = [dists_indices_fst;i];
			dists_indices_snd = [dists_indices_snd;j];
		end
	end

	% sort: 1 for the first dim
	[dists_vec_sorted,dists_vec_perm] = sort(dists_vec);

	% and update the indices
	dists_indices_fst = dists_indices_fst(dists_vec_perm);
	dists_indices_snd = dists_indices_snd(dists_vec_perm);

	% and now, we can begin selecting the indices
	selected = zeros(length_features);
	nr_selected = 0;

	% some arbitrary choices
	% the size is in pairs
	group_size = 1; %; floor(sqrt(length_training_start));
	
	% if we "select" a few more samples it's not a problem
	while(nr_selected <= length_training_start)
		% so basically the choice is with replacement:
		% some groups can intersect others !
		new_indices_begin = randi(length(dists_indices_fst)-group_size);
		for i=new_indices_begin:new_indices_begin+group_size-1
			if (selected(dists_indices_fst(i)) == 0)
				nr_selected = nr_selected + 1;
				selected(dists_indices_fst(i)) = 1;
			end
			if (selected(dists_indices_snd(j)) == 0)
				nr_selected = nr_selected + 1;
				selected(dists_indices_snd(j)) = 1;
			end
		end
	end

	for i=1:length_features
		if (selected(i) == 1)
			selected_samples = [selected_samples;data_array(i,:)];
		else
			rest_data = [rest_data;data_array(i,:)];
		end
	end

	data_array = [selected_samples;rest_data];

end

