% PAL (Pareto Active Learning) Algorithm 
%
% Copyright (c) 2013 ETH Zurich
% 
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

clear all;
fclose('all');
close all;
import java.io.*;
cd gpml-matlab-v3.2-2013-01-15
startup
cd ..

config_file = -1;
while(config_file==-1)
    config_file = input('Input the name of the configuration file: ', 's');
    if exist(config_file,'file')==0
        config_file = -1;
    end
end

%read configuration file
[keynames,values]=textread(config_file,'%s=%s');
v = str2double(values); idx = ~isnan(v);
values(idx) = num2cell(v(idx));
conf = cell2struct(values, keynames); 
conf.epsilon_list = str2num(conf.epsilon_list); 

%make sure that in run mode only one epsilon is taken
if (conf.test_mode == 0)
    conf.epsilon_list = conf.epsilon_list(1);
end

meanfunc = {@meanConst}; 
covfunc = {@covSum, {@covLINone, @covSEard}}; 
likfunc = @likGauss; 

%other variables
gap_hyp_update = 20;
gap_random_sample = 10; %every x iterations a random sample is taken
sampling_strategy=2; % 
plot_prediction = 0; % for debugging the design space can be plotted on every iteration
beta=3; %default value, this will be overwritten
learning_name = 'gp'; %this name is used for the output files
learning_strategy = 2; %use 2 for learning with GP
exchange_file_name = strcat(conf.exchange_path,'/sampling_exchange.csv');
done_request_file_name = strcat(conf.exchange_path,'/done_request');
done_reply_file_name = strcat(conf.exchange_path,'/done_reply');
done_train_file_name = strcat(conf.exchange_path,'/done_train');


if (conf.test_mode == 1)
    file_out_err_obj1 = fopen(strcat(conf.results_folder,'predictions_',learning_name,'_sampl_',num2str(sampling_strategy),'_err_obj1.csv'), 'w');
    file_out_err_obj2 = fopen(strcat(conf.results_folder,'predictions_',learning_name,'_sampl_',num2str(sampling_strategy),'_err_obj2.csv'), 'w');

    file_out_pareto_err_obj1 = fopen(strcat(conf.results_folder,'predictions_',learning_name,'_sampl_',num2str(sampling_strategy),'_pareto_err_obj1.csv'), 'w');
    file_out_pareto_err_obj2 = fopen(strcat(conf.results_folder,'predictions_',learning_name,'_sampl_',num2str(sampling_strategy),'_pareto_err_obj2.csv'), 'w');

    file_out_obj1_hv_class = fopen(strcat(conf.results_folder,'predictions_',learning_name,'_sampl_',num2str(sampling_strategy),'_obj1_hv_class.csv'), 'w');
    file_out_obj2_hv_class = fopen(strcat(conf.results_folder,'predictions_',learning_name,'_sampl_',num2str(sampling_strategy),'_obj2_hv_class.csv'), 'w');

    file_out_stop_obj1 = fopen(strcat(conf.results_folder,'predictions_',learning_name,'_sampl_',num2str(sampling_strategy),'_stop_obj1.csv'), 'w');
    file_out_stop_obj2 = fopen(strcat(conf.results_folder,'predictions_',learning_name,'_sampl_',num2str(sampling_strategy),'_stop_obj2.csv'), 'w');

    file_perf_iter = fopen(strcat(conf.results_folder,'perf_',learning_name,'_sampl_',num2str(sampling_strategy),'_iterations.csv'), 'w');
    file_perf_total = fopen(strcat(conf.results_folder,'perf_',learning_name,'_sampl_',num2str(sampling_strategy),'_total.csv'), 'w');
else
    file_out_train = strcat(conf.results_folder,'predictions_',learning_name,'_sampl_',num2str(sampling_strategy),'_train_set.csv');
    file_out_pareto_predict = fopen(strcat(conf.results_folder,'predictions_',learning_name,'_sampl_',num2str(sampling_strategy),'_pareto_set.csv'), 'w');
    
    exchange_file_name = strcat(conf.exchange_path,'/sampling_exchange.csv');
    done_request_file_name = strcat(conf.exchange_path,'/done_request');
    done_reply_file_name = strcat(conf.exchange_path,'/done_reply');
    done_train_file_name = strcat(conf.exchange_path,'/done_train');
end

%%%%%%%%%load data_all array
string_tmp='';
num_objectives = 2;
if (conf.test_mode == 1)
    cols_to_read = conf.num_features+num_objectives;
    total_cols = conf.num_features+num_objectives+2;
else
    cols_to_read = conf.num_features+2;  %%%%%%%%remove 2 
    total_cols = conf.num_features;
    conf.number_of_repetitions = 1;
end
for i=1:cols_to_read
    string_tmp = strcat(string_tmp,'%f ');
end

data_in_f = fopen(conf.readfile);
data_all = textscan(data_in_f,string_tmp ,'HeaderLines',1,'Delimiter',';','CollectOutput',0);
fclose(data_in_f);

if (conf.test_mode == 1)
    obj1_index = conf.num_features+1;
    obj2_index = conf.num_features+2;
    obj1_original_index = obj2_index + 1;
    obj2_original_index = obj2_index + 2;
    
    if (conf.maximize_obj1 ==1)
        data_all{obj1_index} = invert_data(data_all{obj1_index}); 
    end
    if (conf.maximize_obj2 ==0)
        data_all{obj2_index} = invert_data(data_all{obj2_index});
    end
    
    %keep originals for use in calculating hypervolume error
    data_all{obj1_original_index} = data_all{obj1_index};
    data_all{obj2_original_index} = data_all{obj2_index};

    %normalize objective values
    norm_factor_obj1 = max(data_all{obj1_index})/100;
    norm_factor_obj2 = max(data_all{obj2_index})/100;

    data_all{obj1_index} = normalize_data(data_all{obj1_index});
    data_all{obj2_index} = normalize_data(data_all{obj2_index});
    
    response_obj1 = data_all{obj1_index};
    response_obj2 = data_all{obj2_index};
    response_obj1_original = data_all{obj1_original_index};
    response_obj2_original = data_all{obj2_original_index};
    
    conf.train_data_range_obj1 = max(response_obj1) - min(response_obj1);
    conf.train_data_range_obj2 = max(response_obj2) - min(response_obj2);
    
    real_pareto_set = get_pareto_sets(response_obj1_original, response_obj2_original);
end


%auxiliary variables
n = length(data_all{1});

length_training = uint16((n*conf.max_perc_train)/100);
length_training = cast(length_training,'uint16');
max_train = length_training-conf.length_training_start+1;

for iter2=1:conf.number_of_repetitions
    for epsilon_iter = 1:length(conf.epsilon_list)
			
        %load again features and objectives from data_all
        data_array=[];
        for i =1:total_cols
            data_array=[data_array,data_all{i}];
        end
        
        epsilon = conf.epsilon_list(epsilon_iter);

        %reinitialize variables
		one_not_p = zeros(n,1);
		one_is_pareto = zeros(n,1);
		no_class = ones(n,1);
        sum_pareto = zeros(n,1);

        s_obj1 = zeros(n,1);
        s_obj2 = zeros(n,1);
        var_obj1 = zeros(n,1);
        var_obj2 = zeros(n,1);
        u_rt_obj1 = zeros(n,1);
        u_rt_obj2 = zeros(n,1);
        s_rt_obj1 = zeros(n,1);
        s_rt_obj2 = zeros(n,1);
        
        total_ops = struct('total_ops_class',0,'total_ops_chol',0,'total_ops_mu',0,'total_ops_s2',0);
        total_time = 0;

        epsilon_end = 0;
		start_total_time = cputime;
        response_train_obj1 = -1;
        response_train_obj2 = -1;
        
        for iter_train=1:max_train
            len_train = iter_train+conf.length_training_start-2;
            
            %check if all points are classified
            if sum(epsilon_end)==1
                disp('***************all epsilon ended********************');
                if (conf.test_mode ==0)
			done_file = fopen(done_train_file_name,'w');
                	fprintf(done_file,'done');
                	fclose(done_file);
		end
                break
            end
            
            %reinitialize variables
            start_iteration_time = cputime; 
            iter_ops = struct('ops_class',0,'ops_chol',0,'ops_mu',0,'ops_s2',0);
           
            %calculate the value of beta
            tmp_beta = length(data_all{1})*2*pi^2*(iter_train+conf.length_training_start)^2/(6*conf.delta);
            beta= 2*log(double(tmp_beta))/conf.factor_beta;
           
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%%%%%%%%%%%%%%%%% choose next sample %%%%%%%%%%%%%%%%%%%

            if (iter_train==1)
                %init trainig set
                %bring samples to start to the first rows of data_array
                if (n>1000) 
                    data_array = data_array(randsample(1:length(data_array),length(data_array)),:);
                else
                    data_array = get_start_samples_new(data_array,conf.length_training_start,conf.num_features);                
                end
            else
                if (sampling_strategy == 1)
                    sampling_index = pick_sampling_max_variance(s_obj1+s_obj2);
                elseif (sampling_strategy ==2)
                       
                        if (mod(iter_train,gap_random_sample)==0)
                            %pick a random sample
                            sampling_index = floor(n + (len_train+1-n)*rand);
                            disp('->>>> random sample')
                        else
                            if (conf.s2_optimization==1)
                                s2_updated = zeros(n,1);
                            else
                                s2_updated = ones(n,1);
                            end

                            pre_no_class = no_class | one_is_pareto; %sampling from pareto or unclassified
                            pre_no_class_var = s_rt_obj1 + s_rt_obj2;
                            pre_no_class_var = pre_no_class_var .* pre_no_class;

                            sampling_index_tmp = pick_sampling_max_variance(pre_no_class_var);

                            found_unclassified = 0;
                            found_next_sample = 0;
                            
                            if (sum(no_class(len_train+1:length(no_class)))==0)
                                classify_samples = 1;
                            else
                                classify_samples = 0;
                            end
                            
                            while found_unclassified == 0
                                
                                %update uncertainty region
                                if (s2_updated(sampling_index_tmp)==0)
                                    var_obj1(sampling_index_tmp) = gp_s2(hyp_obj1, @infExact, meanfunc, covfunc, likfunc, features_train, response_train_obj1, data_array(sampling_index_tmp,1:conf.num_features),post_obj1);
                                    var_obj2(sampling_index_tmp) = gp_s2(hyp_obj2, @infExact, meanfunc, covfunc, likfunc, features_train, response_train_obj2, data_array(sampling_index_tmp,1:conf.num_features),post_obj2);
									iter_ops.ops_s2 = iter_ops.ops_s2 + 2 * get_ops_s2(len_train);
                                    s_obj1(sampling_index_tmp) = var_obj1(sampling_index_tmp)^(1/2); 
                                    s_obj2(sampling_index_tmp) = var_obj2(sampling_index_tmp)^(1/2); 
                                    s2_updated(sampling_index_tmp) = 1;

                                    area_a = [u_rt_obj1(sampling_index_tmp),s_rt_obj1(sampling_index_tmp),u_rt_obj2(sampling_index_tmp),s_rt_obj2(sampling_index_tmp)];
                                    area_b = [prediction_obj1(sampling_index_tmp),s_obj1(sampling_index_tmp)*beta,prediction_obj2(sampling_index_tmp),s_obj2(sampling_index_tmp)*beta];
                                    [tmp_a,tmp_b,tmp_c,tmp_d]=get_intersection(area_a,area_b);

                                    u_rt_obj1(sampling_index_tmp) = tmp_a;
                                    s_rt_obj1(sampling_index_tmp) = tmp_b;
                                    u_rt_obj2(sampling_index_tmp) = tmp_c;
                                    s_rt_obj2(sampling_index_tmp) = tmp_d;

                                    pre_no_class_var(sampling_index_tmp) = s_rt_obj1(sampling_index_tmp) + s_rt_obj2(sampling_index_tmp);
                                    sampling_index_update = pick_sampling_max_variance(pre_no_class_var);
                                    if (sampling_index_update~=sampling_index_tmp)
                                        sampling_index_tmp = sampling_index_update;
                                        continue
                                    end
                                end
                                
                                %check if point sampling_index_tmp can be classified:
                                if pre_no_class(sampling_index_tmp)==1
                                    data_range = {conf.train_data_range_obj1,conf.train_data_range_obj2};
                                    cell_out = classify_point(sampling_index_tmp, u_rt_obj1,u_rt_obj2, s_rt_obj1, s_rt_obj2,epsilon,data_range,one_not_p);
									iter_ops.ops_class = iter_ops.ops_class + get_ops_class(n);
                                    one_is_pareto(sampling_index_tmp) = cell_out{1};
                                    one_not_p(sampling_index_tmp) = cell_out{2};
                                end
                                %%sum_s2_updated = sum(s2_updated)
                                
                                if (classify_samples == 0)
                                    if (one_not_p(sampling_index_tmp) == 1)
                                        pre_no_class(sampling_index_tmp)=0;
                                        no_class(sampling_index_tmp)=0;
                                        %update sampling index
                                        pre_no_class_var(sampling_index_tmp) = 0;
                                        sampling_index_tmp = pick_sampling_max_variance(pre_no_class_var);
                                    elseif (one_is_pareto(sampling_index_tmp)==1)
                                        pre_no_class(sampling_index_tmp)=0;
                                        no_class(sampling_index_tmp)=0;
                                        %it has been classified as pareto but continue classifying
                                        if (found_next_sample==0)
                                            found_next_sample = 1;
                                            sampling_index = sampling_index_tmp;
                                            %found next sample - pareto
                                        end
                                        pre_no_class_var(sampling_index_tmp) = 0;
                                        sampling_index_tmp = pick_sampling_max_variance(pre_no_class_var);
                                    else 
                                        if (found_next_sample==0)
                                            sampling_index = sampling_index_tmp;
                                        end
                                        found_unclassified = 1;
                                    end
                                else
                                    %leave sample index, look for unclassified
                                    if (found_next_sample == 0)
                                        sampling_index = floor(n + (len_train+1-n)*rand);
                                        found_next_sample = 1;
                                    end
                                    
                                    sampling_index_tmp = pick_sampling_max_variance(no_class);
                                    
                                    if pre_no_class(sampling_index_tmp)==1
                                        data_range = {conf.train_data_range_obj1,conf.train_data_range_obj2};
                                        cell_out = classify_point(sampling_index_tmp, u_rt_obj1,u_rt_obj2, s_rt_obj1, s_rt_obj2,epsilon,data_range,one_not_p);
                                        iter_ops.ops_class = iter_ops.ops_class + get_ops_class(n);
                                        one_is_pareto(sampling_index_tmp) = cell_out{1};
                                        one_not_p(sampling_index_tmp) = cell_out{2};
                                    end
                                   
                                    if (one_not_p(sampling_index_tmp) == 1)
                                        pre_no_class(sampling_index_tmp)=0;
                                        no_class(sampling_index_tmp)=0;
                                        %update sampling index
                                        pre_no_class_var(sampling_index_tmp) = 0;

                                    elseif (one_is_pareto(sampling_index_tmp)==1)
                                        pre_no_class(sampling_index_tmp)=0;
                                        no_class(sampling_index_tmp)=0;
                                        %it has been classified as pareto but continue classifying
                                        pre_no_class_var(sampling_index_tmp)= 0;
                                        %add ones in points that are not classified
                                    else 
                                        found_unclassified = 1;
                                    end
                                end
                                
                                if ((sum(no_class(len_train+1:length(no_class)))==0) && (classify_samples==0))
                                    classify_samples = 1;
                                    %all points not sampled have been classified
                                    found_next_sample = 0;
                                end
                                
                                sum_unclassified = sum(no_class);
                                if sum_unclassified==0
                                    break
                                end
                            end
                            if sum_unclassified==0
                                    disp('*************** all classified ********************');
                                    epsilon_end = 1;
                            end  
                        end
                else
                    %for random sampling
                    sampling_index = len_train+1;
                end
                
                if (sum(no_class) ~=0)
                    
                	if plot_prediction
                         close all
                         plot_gp_predictions(data_array(:,obj1_index),data_array(:,obj2_index),u_rt_obj1,u_rt_obj2, s_rt_obj1, s_rt_obj2,1,real_pareto_01list,one_not_p,one_is_pareto);                           
                         s_a = s_rt_obj1(sampling_index);
                         s_t = s_rt_obj2(sampling_index);
                         u_a = u_rt_obj1(sampling_index);
                         u_t = u_rt_obj2(sampling_index);
                         obj1_vector = [u_a+s_a,u_a+s_a,u_a-s_a,u_a-s_a,u_a+s_a];
                         obj2_vector = [u_t+s_t,u_t-s_t,u_t-s_t,u_t+s_t,u_t+s_t]; 
                         line(obj1_vector,obj2_vector,'Color','r','LineStyle',':','LineWidth',2);                        
                         disp('check sample')
                         pause
                    end
                    
                    fprintf('sampling index :%d\n',sampling_index);

                    if ((sampling_index<=len_train) && (sum_unclassified~=0))
                        disp('sampling index is in training');
                        pause
                    end

                    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    %%%%%%%%%%%%%%%%%% bring sample to the top of the list %%%%%%%%%%%%%%%%%%%

                    %swap element sampling_index to len_train
                    len_train = iter_train+conf.length_training_start-1;

                    data_array = swap_lines(data_array, sampling_index, len_train);

                    one_not_p = swap_lines(one_not_p, sampling_index, len_train);
                    one_is_pareto = swap_lines(one_is_pareto, sampling_index, len_train);
                    no_class = swap_lines(no_class, sampling_index, len_train);

                    u_rt_obj1 = swap_lines(u_rt_obj1,sampling_index, len_train);
                    u_rt_obj2 = swap_lines(u_rt_obj2,sampling_index, len_train);
                    s_rt_obj1 = swap_lines(s_rt_obj1,sampling_index, len_train);
                    s_rt_obj2 = swap_lines(s_rt_obj2,sampling_index, len_train);
                    s_obj1 = swap_lines(s_obj1,sampling_index, len_train);
                    s_obj2 = swap_lines(s_obj2,sampling_index, len_train);

                end
            end
            
            if sum(no_class)~=0
                len_train = iter_train+conf.length_training_start-1;

                if (conf.test_mode == 1)
                	response_train_obj1 = data_array(1:len_train,obj1_index);
                    response_train_obj2 = data_array(1:len_train,obj2_index); 
                else
                    %%%%get samples when not in test mode
                    
                    %write in file the features of the needed samples
                    exchange_file_tmp = File(exchange_file_name);
                    time_modified1 = exchange_file_tmp.lastModified();

                    exchange_file = fopen(exchange_file_name, 'w');
                    
                    features_request = -1;
                    if iter_train == 1
                        %read a set of sampling points
                        for j = 1:conf.length_training_start
                            for i =1:total_cols-1
                                fprintf(exchange_file, '%f;',data_array(j,i));
                            end
                            
                            fprintf(exchange_file, '%f\n',data_array(j,total_cols));
                            features_request = append(features_request,data_array(j,:));
                        end
                    else
                        for i =1:total_cols-1
                            fprintf(exchange_file, '%f;',data_array(len_train,i));
                        end
                        fprintf(exchange_file, '%f\n',data_array(len_train,total_cols));
                        features_request = append(features_request,data_array(len_train,:));
                    end
                    
                    %print training set in file
                    fclose(data_in_f);
                    done_file = fopen(done_request_file_name,'w');
                    fprintf(done_file,'done');
                    fclose(done_file);
                    
                    while (exist(done_reply_file_name,'file')==0)
                        1;
                    end
                    system(['rm ',done_reply_file_name]);
                    exchange_file = fopen(exchange_file_name, 'r');
                    %read a set of sampling points
                        
                    for i=1:conf.num_features+num_objectives+2
                        string_tmp = strcat(string_tmp,'%f ');
                    end

                    data_train_in = textscan(exchange_file,string_tmp ,'HeaderLines',0,'Delimiter',';','CollectOutput',0);
                    fclose(data_in_f);  
                    obj1_in = data_train_in{conf.num_features+1};
                    obj2_in = data_train_in{conf.num_features+2};

                    if (conf.maximize_obj1 == 1)
                        obj1_in = invert_data(obj1_in); 
                    end
                    if (conf.maximize_obj2 == 0)
                        obj2_in = invert_data(obj2_in);
                    end

                    response_train_obj1 = append(response_train_obj1,obj1_in);
                    response_train_obj2 = append(response_train_obj2,obj2_in);

                    out_array_tmp = [features_request,data_train_in{conf.num_features+1},data_train_in{conf.num_features+2}];
                    dlmwrite(file_out_train,out_array_tmp,'-append');
  
                end
                            
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %%%%%%%%%%%%%%%%%% update models and make prediction %%%%%%%%%%%%%%%%%%%

                %split array (train and test data)
                features_train = data_array(1:len_train,1:conf.num_features);
                features_test = data_array(len_train+1:length(data_array),1:conf.num_features);

                if (learning_strategy==1)
                    %decision trees
                    [prediction_test_obj1, var_test_obj1] = tree_variance(features_train, response_train_obj1, features_test);
                    [prediction_test_obj2, var_test_obj2] = tree_variance(features_train, response_train_obj2, features_test);

                elseif (learning_strategy==2)
                    %GP regression
                    if ((iter_train==1) || mod(iter_train,gap_hyp_update)==0)
                            disp('find hyperparams........')
                            hyp_obj1  = gp_find_hp(features_train, response_train_obj1);
                            hyp_obj2 = gp_find_hp(features_train, response_train_obj2);

					        [del1,del2,post_obj1] = gp(hyp_obj1, @infExact, meanfunc, covfunc, likfunc, features_train, response_train_obj1);
							prediction_test_obj1 = gp_mu(hyp_obj1, @infExact, meanfunc, covfunc, likfunc, features_train, response_train_obj1, features_test,post_obj1);
                            var_test_obj1 = gp_s2(hyp_obj1, @infExact, meanfunc, covfunc, likfunc, features_train, response_train_obj1,features_test,post_obj1);
							
                            [del1,del2,post_obj2] = gp(hyp_obj2, @infExact, meanfunc, covfunc, likfunc, features_train, response_train_obj2);
                            prediction_test_obj2 = gp_mu(hyp_obj2, @infExact, meanfunc, covfunc, likfunc, features_train, response_train_obj2, features_test,post_obj2);
                            var_test_obj2 = gp_s2(hyp_obj2, @infExact, meanfunc, covfunc, likfunc, features_train, response_train_obj2,features_test,post_obj2);
                            var_obj1 = [zeros(length(response_train_obj1),1);var_test_obj1];
                            var_obj2 = [zeros(length(response_train_obj2),1);var_test_obj2];
                            s_obj1 = var_obj1.^(1/2); 
                            s_obj2 = var_obj2.^(1/2); 
                            
                            iter_ops.ops_chol = iter_ops.ops_chol + 2 * get_ops_chol(len_train);
                            iter_ops.ops_mu = iter_ops.ops_mu + 2 * get_ops_mu(n,len_train);
							iter_ops.ops_s2 = iter_ops.ops_s2 + 2 * (n-len_train) * get_ops_s2(len_train);
                    else
                        try
                            [del1,del2,post_obj1] = gp(hyp_obj1, @infExact, meanfunc, covfunc, likfunc, features_train, response_train_obj1);
                            prediction_test_obj1 = gp_mu(hyp_obj1, @infExact, meanfunc, covfunc, likfunc, features_train, response_train_obj1, features_test,post_obj1);
                            if (conf.s2_optimization == 0)
                                var_test_obj1 = gp_s2(hyp_obj1, @infExact, meanfunc, covfunc, likfunc, features_train, response_train_obj1,features_test,post_obj1);
                                iter_ops.ops_s2 = iter_ops.ops_s2 + (n-len_train) * get_ops_s2(len_train);
                            end
                            iter_ops.ops_chol = iter_ops.ops_chol + get_ops_chol(len_train);
                            iter_ops.ops_mu = iter_ops.ops_mu + get_ops_mu(n,len_train);

                        catch
                            hyp_obj1  = gp_find_hp(features_train, response_train_obj1);
                            [del1,del2,post_obj1] = gp(hyp_obj1, @infExact, meanfunc, covfunc, likfunc, features_train, response_train_obj1);
                            prediction_test_obj1 = gp_mu(hyp_obj1, @infExact, meanfunc, covfunc, likfunc, features_train, response_train_obj1, features_test,post_obj1);
                            disp('recalculate hyperparams...')
                            if (conf.s2_optimization == 0)
                                var_test_obj1 = gp_s2(hyp_obj1, @infExact, meanfunc, covfunc, likfunc, features_train, response_train_obj1,features_test,post_obj1);
                                iter_ops.ops_s2 = iter_ops.ops_s2 + (n-len_train) * get_ops_s2(len_train);
                            end
                            iter_ops.ops_chol = (iter_ops.ops_chol + get_ops_chol(len_train));
                            iter_ops.ops_mu = iter_ops.ops_mu + get_ops_mu(n,len_train);
                        end  
                        try
                            hyp_obj2.cov = hyp_obj2.cov;
                            hyp_obj2.lik = hyp_obj2.lik;
                            hyp_obj2.mean = hyp_obj2.mean;
                            [del1,del2,post_obj2] = gp(hyp_obj2, @infExact, meanfunc, covfunc, likfunc, features_train, response_train_obj2);
                            prediction_test_obj2 = gp_mu(hyp_obj2, @infExact, meanfunc, covfunc, likfunc, features_train, response_train_obj2, features_test,post_obj2);
                            if (conf.s2_optimization == 0)
                                var_test_obj2 = gp_s2(hyp_obj2, @infExact, meanfunc, covfunc, likfunc, features_train, response_train_obj2,features_test,post_obj2);
                                iter_ops.ops_s2 = iter_ops.ops_s2 + (n-len_train) * get_ops_s2(len_train);
                            end
                            iter_ops.ops_mu = iter_ops.ops_mu + get_ops_mu(n,len_train);
							iter_ops.ops_chol = (iter_ops.ops_chol + get_ops_chol(len_train));
                        catch
                             hyp_obj2 = gp_find_hp(features_train, response_train_obj2);
                             [del1,del2,post_obj2] = gp(hyp_obj2, @infExact, meanfunc, covfunc, likfunc, features_train, response_train_obj2);
                             prediction_test_obj2 = gp_mu(hyp_obj2, @infExact, meanfunc, covfunc, likfunc, features_train, response_train_obj2, features_test,post_obj2);
                             disp('recalculate hyperparams...')  
                             if (conf.s2_optimization == 0)
                                var_test_obj2 = gp_s2(hyp_obj2, @infExact, meanfunc, covfunc, likfunc, features_train, response_train_obj2,features_test,post_obj2);
                                iter_ops.ops_s2 = iter_ops.ops_s2 + (n-len_train) * get_ops_s2(len_train);
                             end
                             iter_ops.ops_mu = iter_ops.ops_mu + get_ops_mu(n,len_train);
                             iter_ops.ops_chol = (iter_ops.ops_chol + get_ops_chol(len_train));
                        end
                    end
                end
                
                if (conf.s2_optimization == 0)
                    var_obj1 = [zeros(length(response_train_obj1),1);var_test_obj1];
                    var_obj2 = [zeros(length(response_train_obj2),1);var_test_obj2];
                    s_obj1 = var_obj1.^(1/2); 
                    s_obj2 = var_obj2.^(1/2); 
                end

                prediction_obj1 = [response_train_obj1;prediction_test_obj1];
                prediction_obj2 = [response_train_obj2;prediction_test_obj2];
                s_obj1(len_train) = 0;
                s_obj2(len_train) = 0;

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %%%%%%%%%%%%%%%%%% intersect confidence regions %%%%%%%%%%%%%%%%%%%
                if (iter_train>1)
                    for iter_k = 1:length(s_obj1)
                        %using new mean and old std dev
                        area_a = [u_rt_obj1(iter_k),s_rt_obj1(iter_k),u_rt_obj2(iter_k),s_rt_obj2(iter_k)];
                        area_b = [prediction_obj1(iter_k),s_obj1(iter_k)*beta,prediction_obj2(iter_k),s_obj2(iter_k)*beta];
                        [tmp_a,tmp_b,tmp_c,tmp_d]=get_intersection(area_a,area_b);

                        u_rt_obj1(iter_k) = tmp_a;
                        s_rt_obj1(iter_k) = tmp_b;
                        u_rt_obj2(iter_k) = tmp_c;
                        s_rt_obj2(iter_k) = tmp_d;
                    end
                else
                   u_rt_obj1 = prediction_obj1;
                   s_rt_obj1 = s_obj1*beta;
                   u_rt_obj2 = prediction_obj2;
                   s_rt_obj2 = s_obj2*beta;
                end
            end

			total_time_iteration = cputime - start_iteration_time; 
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%%%%%%%%%%%%%%%%% calculate hypervolume error %%%%%%%%%%%%%%%%%%%
            if (conf.test_mode == 0)
                continue
            end
            
            if sum(no_class)~=0
                %calculate prediction errors (on all points and on real-Pareto points)
                response_test_obj1 = data_array(len_train+1:length(data_array),obj1_index);           
                response_test_obj2 = data_array(len_train+1:length(data_array),obj2_index);
                avg_error_obj1 = get_prediction_avg_error(response_test_obj1*norm_factor_obj1,prediction_test_obj1*norm_factor_obj1,conf.log_train);
                avg_error_obj2 = get_prediction_avg_error(response_test_obj2*norm_factor_obj2,prediction_test_obj2*norm_factor_obj2,conf.log_train);

                real_pareto_01list = get_pareto_01list(data_array(:,obj1_index),data_array(:,obj2_index));
                avg_error_pareto_obj1 = get_prediction_avg_error_pareto(data_array(:,obj1_index)*norm_factor_obj1,u_rt_obj1*norm_factor_obj1,real_pareto_01list,conf.log_train);
                avg_error_pareto_obj2 = get_prediction_avg_error_pareto(data_array(:,obj2_index)*norm_factor_obj2,u_rt_obj2*norm_factor_obj2,real_pareto_01list,conf.log_train);

                predicted_pareto_01list = get_pareto_01list(prediction_obj1, prediction_obj2);
                %plus sampled points
                predicted_pareto_01list = predicted_pareto_01list | [ones(len_train,1);zeros(n-len_train,1)];

                [avg_pareto_hv_error_obj1,avg_pareto_hv_error_obj2] = get_pareto_hv_error(real_pareto_set,predicted_pareto_01list,data_array(:,obj1_original_index),data_array(:,obj2_original_index),conf.log_train);
            end
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%%%%%%%%%%%%%%%%% Final stats and print outs %%%%%%%%%%%%%%%%%%%         
                    
            if (plot_prediction==1)
            	close all
             	plot_gp_predictions(data_array(:,obj1_index),data_array(:,obj2_index),u_rt_obj1,u_rt_obj2, s_rt_obj1, s_rt_obj2,1,real_pareto_01list,one_not_p,one_is_pareto);                           
            end 
                     
            not_p_not_not_p = n - sum(one_is_pareto|one_not_p);
            perc_not_catheg = (round((not_p_not_not_p/n)*100)); %%%%%%check this, set it to floor
            number_executions = sum(predicted_pareto_01list(len_train:end)) + len_train;

			total_ops.total_ops_class = total_ops.total_ops_class + iter_ops.ops_class;				
			total_ops.total_ops_chol = total_ops.total_ops_chol + iter_ops.ops_chol;				
			total_ops.total_ops_mu = total_ops.total_ops_mu + iter_ops.ops_mu;				
			total_ops.total_ops_s2 = total_ops.total_ops_s2 + iter_ops.ops_s2;			
					
            fprintf(file_out_obj1_hv_class, '%f,%d,%d,%f\n',epsilon,len_train,number_executions,avg_pareto_hv_error_obj1);
            fprintf(file_out_obj2_hv_class, '%f,%d,%d,%f\n',epsilon,len_train,number_executions,avg_pareto_hv_error_obj2);

            if (epsilon_end==1 || iter_train==max_train)
                fprintf(file_out_stop_obj1, '%f,%d,%d,%f\n',epsilon,len_train,number_executions,avg_pareto_hv_error_obj1);
                fprintf(file_out_stop_obj2, '%f,%d,%d,%f\n',epsilon,len_train,number_executions,avg_pareto_hv_error_obj2);
                            
				total_time = cputime - start_total_time;
				fprintf(file_perf_total, '%d,%d,%d,%d,%f\n',total_ops.total_ops_class,total_ops.total_ops_chol,total_ops.total_ops_mu,total_ops.total_ops_s2,total_time);    
            end
                   
			perc_train = len_train*100/n;

            fprintf(file_out_err_obj1, '%d,%f,%d\n',perc_train,avg_error_obj1,len_train);
            fprintf(file_out_err_obj2, '%d,%f,%d\n',perc_train,avg_error_obj2,len_train);
            fprintf(file_out_pareto_err_obj1, '%d,%f,%d\n',perc_train,avg_error_pareto_obj1,len_train);
            fprintf(file_out_pareto_err_obj2, '%d,%f,%d\n',perc_train,avg_error_pareto_obj2,len_train);
                
            disp(sprintf('\npercentage not classified: %d',perc_not_catheg));
            disp(sprintf('percentage train: %d. Evaluations: %d\n',perc_train,len_train));
            disp(sprintf('error pareto: %f,%f\n', avg_error_pareto_obj1, avg_error_pareto_obj2));
            disp(sprintf('error all: %f,%f.\n', avg_error_obj1, avg_error_obj2))
            disp(sprintf('hypervolume error: %f,%f. With %d samples evaluated', avg_pareto_hv_error_obj1, avg_pareto_hv_error_obj2,number_executions))

        	fprintf(file_perf_iter, '%d,%d,%d,%d,%f,%f\n',iter_ops.ops_class,iter_ops.ops_chol,iter_ops.ops_mu,iter_ops.ops_s2,total_time_iteration,total_time);
           
        end
        disp('**** ended *****')

    end

end
